;; Friday theme 0.1
;; Designed & implemented by Stefan Zeiger <szeiger@szeiger.de>

(defgroup Friday "Friday Theme" :group appearance)

(defcustom Friday:inactive-font-color "#666660"
  "Text color on title bars of inactive windows."
  :type color
  :group (appearance Friday)
  :after-set after-setting-frame-option)

(defcustom Friday:focused-font-color "#ffffff"
  "Text color on title bars of focused windows."
  :type color
  :group (appearance Friday)
  :after-set after-setting-frame-option)

(defcustom Friday:show-menu-icons t
  "Show window icon on the menu button."
  :type boolean
  :group (appearance Friday)
  :after-set after-setting-frame-option)

(let* (
     (title-width (lambda (w)
       (let ((w-width (car (window-dimensions w))))
         (max 0 (min (- w-width 65) (text-width (window-name w)))))
     ))

     (menu-image (lambda (w)
       (if Friday:show-menu-icons (window-icon-image w) nil)
     ))

     (make-button-images (lambda (name)
       (let ( (i-img  (make-image (concat "button-" name "-i.png")))
              (f-img  (make-image (concat "button-" name "-f.png")))
              (ih-img (make-image (concat "button-" name "-ih.png")))
              (h-img  (make-image (concat "button-" name "-h.png")))
              (c-img  (make-image (concat "button-" name "-c.png"))) )
         (list
           (cons 'inactive             i-img)
           (cons 'focused              f-img)
           (cons 'inactive-highlighted ih-img)
           (cons 'highlighted          h-img)
           (cons 'inactive-clicked     c-img)
           (cons 'clicked              c-img)
     ))))

     (make-border-image (lambda (name l-off r-off t-off b-off)
       (let ((img (make-image name)))
         (progn
           (set-image-border img l-off r-off t-off b-off)
           img
     ))))

     (font-colors (lambda (w) (list Friday:inactive-font-color
                                    Friday:focused-font-color)))

     (close-images (make-button-images "c"))

     (close-small-images (make-button-images "cs"))

     (iconify-images (make-button-images "i"))

     (max-restore-images (make-button-images "mr"))

     (menu-images (make-button-images "m"))

     (top-left-images (list (make-image "tl-i.png")
                            (make-image "tl-f.png")))

     (top-right-images (list (make-image "tr-i.png")
                             (make-image "tr-f.png")))

     (top-right-oblique-images (list (make-image "tr-oblique-i.png")
                                     (make-image "tr-oblique-f.png")))

     (bottom-left-images (list (make-image "bl-i.png")
                               (make-image "bl-f.png")))

     (bottom-right-images (list (make-image "br-i.png")
                                (make-image "br-f.png")))

     (bottom-images (list (make-border-image "b-i.png" 1 1 0 0)
                          (make-border-image "b-f.png" 1 1 0 0)))

     (left-images (list (make-border-image "l-i.png" 0 0 7 1)
                        (make-border-image "l-f.png" 0 0 7 1)))

     (right-images (list (make-border-image "r-i.png" 0 0 1 1)
                         (make-border-image "r-f.png" 0 0 1 1)))

     (ltitle-images (list (make-border-image "ltitle-i.png" 14 0 0 0)
                          (make-border-image "ltitle-f.png" 14 0 0 0)))

     (rtitle-images (list (make-border-image "rtitle-i.png" 9 35 0 0)
                          (make-border-image "rtitle-f.png" 9 35 0 0)))

     (shaped-ltitle-images (list
         (make-border-image "shaped-ltitle-i.png" 15 0 0 0)
         (make-border-image "shaped-ltitle-f.png" 15 0 0 0)))

     (shaped-rtitle-images (list
         (make-border-image "shaped-rtitle-i.png" 11 27 0 0)
         (make-border-image "shaped-rtitle-f.png" 11 27 0 0)))

     (shaded-ltitle-images (list
         (make-border-image "shaded-ltitle-i.png" 18 0 0 0)
         (make-border-image "shaded-ltitle-f.png" 18 1 0 0)))

     (shaded-rtitle-images (list
         (make-border-image "shaded-rtitle-i.png" 12 31 0 0)
         (make-border-image "shaded-rtitle-f.png" 12 31 0 0)))

     (ltop-images (list (make-border-image "ltop-i.png" 6 0 0 0)
                        (make-border-image "ltop-f.png" 6 0 0 0)))

     (rtop-images (list (make-border-image "rtop-i.png" 11 19 0 0)
                        (make-border-image "rtop-f.png" 11 19 0 0)))

     (frame `(
       ((class . top-border)
        (left-edge . 8)
        (top-edge . -20)
        (width . ,(lambda (w) (+ (title-width w) 12)))
        (background . ,ltop-images))

       ((class . top-border)
        (right-edge . 8)
        (top-edge . -20)
        (left-edge . ,(lambda (w) (+ (title-width w) 20)))
        (background . ,rtop-images))

       ((class . title)
        (left-edge . 0)
        (top-edge . -17)
        (width . ,(lambda (w) (+ (title-width w) 20)))
        (text . ,window-name)
        (foreground . ,font-colors)
        (x-justify . 18)
        (y-justify . 0)
        (background . ,ltitle-images))

       ((class . title)
        (right-edge . 0)
        (left-edge . ,(lambda (w) (+ (title-width w) 20)))
        (top-edge . -16)
        (background . ,rtitle-images))

       ((class . menu-button)
        (top-edge . -16)
        (cursor . left_ptr)
        (left-edge . 0)
        (foreground . ,menu-image)
        (scale-foreground . t)
        (background . ,menu-images))

       ((class . close-button)
        (removable . t)
        (top-edge . -16)
        (cursor . left_ptr)
        (right-edge . 0)
        (background . ,close-images))

       ((class . iconify-button)
        (removable . t)
        (top-edge . -19)
        (cursor . left_ptr)
        (right-edge . 8)
        (background . ,iconify-images))

       ((class . maximize-button)
        (removable . t)
        (top-edge . -18)
        (cursor . left_ptr)
        (right-edge . 17)
        (background . ,max-restore-images))

       ((class . bottom-border)
        (left-edge . 8)
        (right-edge . 8)
        (bottom-edge . -4)
        (background . ,bottom-images))

       ((left-edge . -4)
        (class . top-left-corner)
        (top-edge . -20)
        (background . ,top-left-images))

       ((right-edge . -4)
        (class . top-right-corner)
        (top-edge . -20)
        (background . ,top-right-oblique-images))

       ((left-edge . -4)
        (top-edge . -8)
        (class . left-border)
        (bottom-edge . 8)
        (background . ,left-images))

       ((class . right-border)
        (right-edge . -4)
        (top-edge . -8)
        (bottom-edge . 8)
        (background . ,right-images))

       ((class . bottom-left-corner)
        (left-edge . -4)
        (bottom-edge . -4)
        (background . ,bottom-left-images))

       ((class . bottom-right-corner)
        (right-edge . -4)
        (bottom-edge . -4)
        (background . ,bottom-right-images))))

     (transient-frame `(
       ((class . top-border)
        (left-edge . 8)
        (right-edge . 8)
        (top-edge . -4)
        (background . ,bottom-images))

       ((class . bottom-border)
        (left-edge . 8)
        (right-edge . 8)
        (bottom-edge . -4)
        (background . ,bottom-images))

       ((left-edge . -4)
        (class . top-left-corner)
        (top-edge . -4)
        (background . ,top-left-images))

       ((right-edge . -4)
        (class . top-right-corner)
        (top-edge . -4)
        (background . ,top-right-images))

       ((left-edge . -4)
        (top-edge . 8)
        (class . left-border)
        (bottom-edge . 8)
        (background . ,right-images))

       ((class . right-border)
        (right-edge . -4)
        (top-edge . 8)
        (bottom-edge . 8)
        (background . ,right-images))

       ((class . bottom-left-corner)
        (left-edge . -4)
        (bottom-edge . -4)
        (background . ,bottom-left-images))

       ((class . bottom-right-corner)
        (right-edge . -4)
        (bottom-edge . -4)
        (background . ,bottom-right-images))))

     (shaped-frame `(
       ((class . title)
        (left-edge . -1)
        (top-edge . -20)
        (width . ,(lambda (w) (+ (title-width w) 21)))
        (text . ,window-name)
        (foreground . ,font-colors)
        (x-justify . 19)
        (y-justify . 3)
        (background . ,shaped-ltitle-images))

       ((class . title)
        (right-edge . -1)
        (left-edge . ,(lambda (w) (+ (title-width w) 20)))
        (top-edge . -20)
        (background . ,shaped-rtitle-images))

       ((class . menu-button)
        (top-edge . -16)
        (cursor . left_ptr)
        (left-edge . 0)
        (foreground . ,menu-image)
        (scale-foreground . t)
        (background . ,menu-images))

       ((class . close-button)
        (removable . t)
        (top-edge . -16)
        (cursor . left_ptr)
        (right-edge . 0)
        (background . ,close-images))

       ((class . iconify-button)
        (removable . t)
        (top-edge . -19)
        (cursor . left_ptr)
        (right-edge . 8)
        (background . ,iconify-images))

       ((class . maximize-button)
        (removable . t)
        (top-edge . -18)
        (cursor . left_ptr)
        (right-edge . 17)
        (background . ,max-restore-images))))

     (shaded-frame `(
       ((class . title)
        (left-edge . -4)
        (top-edge . -20)
        (width . ,(lambda (w) (+ (title-width w) 24)))
        (text . ,window-name)
        (foreground . ,font-colors)
        (x-justify . 22)
        (y-justify . 3)
        (background . ,shaded-ltitle-images))

       ((class . title)
        (right-edge . -4)
        (left-edge . ,(lambda (w) (+ (title-width w) 20)))
        (top-edge . -20)
        (background . ,shaded-rtitle-images))

       ((class . menu-button)
        (top-edge . -16)
        (cursor . left_ptr)
        (left-edge . 0)
        (foreground . ,menu-image)
        (scale-foreground . t)
        (background . ,menu-images))

       ((class . close-button)
        (removable . t)
        (top-edge . -16)
        (cursor . left_ptr)
        (right-edge . 0)
        (background . ,close-small-images))

       ((class . iconify-button)
        (removable . t)
        (top-edge . -19)
        (cursor . left_ptr)
        (right-edge . 8)
        (background . ,iconify-images))

       ((class . maximize-button)
        (removable . t)
        (top-edge . -18)
        (cursor . left_ptr)
        (right-edge . 17)
        (background . ,max-restore-images))))
    )

  (add-frame-style 'Friday
    (lambda (w type)
      (cond ((eq type 'shaped)           shaped-frame)
            ((eq type 'shaded)           shaded-frame)
            ((eq type 'transient)        transient-frame)
            ((eq type 'shaped-transient) shaped-frame)
            ((eq type 'shaded-transient) shaded-frame)
            ((eq type 'unframed)         nil-frame)
            (t                           frame))))

  (call-after-property-changed 'WM_NAME
    (lambda () (rebuild-frames-with-style 'Friday)))

)
